@@ -2,6 +2,8 @@
 #this repository contains the full copyright notices and license terms.
 from trytond.model import ModelSQL, ModelView, fields
 from trytond.tools import Cache, reduce_ids
+from trytond.backend import TableHandler
+from trytond.pyson import If, Bool, Not, Eval, Greater
 import uuid
 import vobject
 import dateutil.tz
@@ -69,6 +71,7 @@ class Calendar(ModelSQL, ModelView):
                 return False
         return True
 
+    @Cache('calendar_calendar.get_name')
     def get_name(self, cursor, user, name, context=None):
         '''
         Return the calendar id of the name
@@ -86,8 +89,6 @@ class Calendar(ModelSQL, ModelView):
             return calendar_ids[0]
         return False
 
-    get_name = Cache('calendar_calendar.get_name')(get_name)
-
     def calendar2ical(self, cursor, user, calendar_id, context=None):
         '''
         Return an iCalendar object for the given calendar_id containing
@@ -286,6 +287,7 @@ class Calendar(ModelSQL, ModelView):
         :param context: the context
         :return: the xml with schedule-response
         '''
+        from DAV.errors import DAV_Forbidden
         collection_obj = self.pool.get('webdav.collection')
 
         calendar_id = collection_obj.calendar(cursor, user, uri,
@@ -432,7 +434,7 @@ class Event(ModelSQL, ModelView):
     uuid = fields.Char('UUID', required=True,
             help='Universally Unique Identifier', select=1)
     calendar = fields.Many2One('calendar.calendar', 'Calendar',
-            required=True, select=1)
+            required=True, select=1, ondelete="CASCADE")
     summary = fields.Char('Summary')
     sequence = fields.Integer('Sequence')
     description = fields.Text('Description')
@@ -455,7 +457,7 @@ class Event(ModelSQL, ModelView):
         ('cancelled', 'Cancelled'),
         ], 'Status')
     organizer = fields.Char('Organizer', states={
-        'required': "bool(attendees) and not bool(parent)",
+        'required': If(Bool(Eval('attendees')), Not(Bool(Eval('parent'))), False),
         }, depends=['attendees', 'parent'])
     attendees = fields.One2Many('calendar.event.attendee', 'event',
             'Attendees')
@@ -466,46 +468,45 @@ class Event(ModelSQL, ModelView):
     alarms = fields.One2Many('calendar.event.alarm', 'event', 'Alarms')
     rdates = fields.One2Many('calendar.event.rdate', 'event', 'Recurrence Dates',
             states={
-                'invisible': "bool(parent)",
+                'invisible': Bool(Eval('parent')),
             }, depends=['parent'])
     rrules = fields.One2Many('calendar.event.rrule', 'event', 'Recurrence Rules',
             states={
-                'invisible': "bool(parent)",
+                'invisible': Bool(Eval('parent')),
             }, depends=['parent'])
     exdates = fields.One2Many('calendar.event.exdate', 'event', 'Exception Dates',
             states={
-                'invisible': "bool(parent)",
+                'invisible': Bool(Eval('parent')),
             }, depends=['parent'])
     exrules = fields.One2Many('calendar.event.exrule', 'event', 'Exception Rules',
             states={
-                'invisible': "bool(parent)",
+                'invisible': Bool(Eval('parent')),
             }, depends=['parent'])
     occurences = fields.One2Many('calendar.event', 'parent', 'Occurences',
-            domain=["('uuid', '=', uuid)",
-                "('calendar', '=', calendar)"],
+            domain=[
+                ('uuid', '=', Eval('uuid')),
+                ('calendar', '=', Eval('calendar')),
+            ],
             states={
-                'invisible': "bool(parent)",
+                'invisible': Bool(Eval('parent')),
             }, depends=['uuid', 'calendar', 'parent'])
     parent = fields.Many2One('calendar.event', 'Parent',
-            domain=["('uuid', '=', uuid)",
-                "('parent', '=', False)",
-                "('calendar', '=', calendar)"],
+            domain=[
+                ('uuid', '=', Eval('uuid')),
+                ('parent', '=', False),
+                ('calendar', '=', Eval('calendar')),
+            ],
             ondelete='CASCADE', depends=['uuid', 'calendar'])
     recurrence = fields.DateTime('Recurrence', select=1, states={
-                'invisible': "not bool(parent)",
+                'invisible': Not(Bool(Eval('_parent_parent'))),
+                'required': Bool(Eval('_parent_parent')),
                 }, depends=['parent'])
-    calendar_owner = fields.Function('get_calendar_field',
-            type='many2one', relation='res.user', string='Owner',
-            fnct_search='search_calendar_field')
-    calendar_read_users = fields.Function('get_calendar_field',
-            type='many2many', relation='res.user', string='Read Users',
-            fnct_search='search_calendar_field')
-    calendar_write_users = fields.Function('get_calendar_field',
-            type='many2many', relation='res.user', string='Write Users',
-            fnct_search='search_calendar_field')
-    classification_public = fields.Function('get_classification_public',
-            type='boolean', string='Classification Public',
-            fnct_search='search_classification_public')
+    calendar_owner = fields.Function(fields.Many2One('res.user', 'Owner'),
+            'get_calendar_field', searcher='search_calendar_field')
+    calendar_read_users = fields.Function(fields.Many2One('res.user',
+        'Read Users'), 'get_calendar_field', searcher='search_calendar_field')
+    calendar_write_users = fields.Function(fields.One2Many('res.user', None,
+        'Write Users'), 'get_calendar_field', searcher='search_calendar_field')
     vevent = fields.Binary('vevent')
 
     def __init__(self):
@@ -521,6 +522,20 @@ class Event(ModelSQL, ModelView):
             'invalid_recurrence': 'Recurrence can not be recurrent!',
         })
 
+    def init(self, cursor, module_name):
+        # Migrate from 1.4: remove classification_public
+        model_data_obj = self.pool.get('ir.model.data')
+        rule_obj = self.pool.get('ir.rule')
+        model_data_ids = model_data_obj.search(cursor, 0, [
+            ('fs_id', '=', 'rule_group_read_calendar_line3'),
+            ('module', '=', module_name),
+            ('inherit', '=', False),
+            ], limit=1)
+        if model_data_ids:
+            model_data = model_data_obj.browse(cursor, 0, model_data_ids[0])
+            rule_obj.delete(cursor, 0, model_data.db_id)
+        return super(Event, self).init(cursor, module_name)
+
     def default_uuid(self, cursor, user, context=None):
         return str(uuid.uuid4())
 
@@ -541,7 +556,7 @@ class Event(ModelSQL, ModelView):
     def timezones(self, cursor, user, context=None):
         return [(x, x) for x in pytz.common_timezones] + [('', '')]
 
-    def get_calendar_field(self, cursor, user, ids, name, arg, context=None):
+    def get_calendar_field(self, cursor, user, ids, name, context=None):
         assert name in ('calendar_owner', 'calendar_read_users',
                 'calendar_write_users'), 'Invalid name'
         res = {}
@@ -553,35 +568,8 @@ class Event(ModelSQL, ModelView):
                 res[event.id] = event.calendar[name].id
         return res
 
-    def search_calendar_field(self, cursor, user, name, args, context=None):
-        args2 = []
-        i = 0
-        while i < len(args):
-            field = args[i][0][9:]
-            args2.append(tuple(['calendar.' + field] + list(args[i])[1:]))
-            i += 1
-        return args2
-
-    def get_classification_public(self, cursor, user, ids, name, arg,
-            context=None):
-        res = {}
-        for event in self.browse(cursor, user, ids, context=context):
-            res[event.id] = False
-            if event.classification == 'public':
-                res[event.id] = True
-        return res
-
-    def search_classification_public(self, cursor, user, name, args,
-            context=None):
-        args2 = []
-        i = 0
-        while i < len(args):
-            if args[i][2]:
-                args2.append(('classification', '=', 'public'))
-            else:
-                args2.append(('classification', '!=', 'public'))
-            i += 1
-        return args2
+    def search_calendar_field(self, cursor, user, name, clause, context=None):
+        return [('calendar.' + name[9:],) + clause[1:]]
 
     def check_recurrence(self, cursor, user, ids):
         '''
@@ -890,7 +878,8 @@ class Event(ModelSQL, ModelView):
         if hasattr(vevent, 'recurrence-id'):
             if not isinstance(vevent.recurrence_id.value, datetime.datetime):
                 res['recurrence'] = datetime.datetime.combine(
-                        vevent.recurrence_id.value, datetime.time())
+                        vevent.recurrence_id.value, datetime.time()
+                        ).replace(tzinfo=tzlocal)
             else:
                 if vevent.recurrence_id.value.tzinfo:
                     res['recurrence'] = \
@@ -1047,14 +1036,14 @@ class Event(ModelSQL, ModelView):
             occurences_todel = [x.id for x in event.occurences]
         for vevent in vevents:
             event_id = None
-            if event:
-                for occurence in event.occurences:
-                    if occurence.recurrence.replace(tzinfo=tzlocal) \
-                            == vevent.recurrence_id.value:
-                        event_id = occurence.id
-                        occurences_todel.remove(occurence.id)
             vals = self.ical2values(cursor, user, event_id, ical,
                     calendar_id, vevent=vevent, context=context)
+            if event:
+                for occurence in event.occurences:
+                    if vals['recurrence'] == \
+                            occurence.recurrence.replace(tzinfo=tzlocal):
+                        event_id = occurence.id
+                        occurences_todel.remove(occurence.id)
             if event:
                 vals['uuid'] = event.uuid
             else:
@@ -1066,7 +1055,7 @@ class Event(ModelSQL, ModelView):
                 res['occurences'].append(('create', vals))
         if occurences_todel:
             res.setdefault('occurences', [])
-            res['occurences'].append(('delete', occurences_todel))
+            res['occurences'].insert(0, ('delete', occurences_todel))
         return res
 
     def event2ical(self, cursor, user, event, context=None):
@@ -1146,7 +1135,7 @@ class Event(ModelSQL, ModelView):
         if not hasattr(vevent, 'last-modified'):
             vevent.add('last-modified')
         vevent.last_modified.value = date.replace(tzinfo=tzlocal)
-        if event.recurrence:
+        if event.recurrence and event.parent:
             if not hasattr(vevent, 'recurrence-id'):
                 vevent.add('recurrence-id')
             if event.all_day:
@@ -1154,6 +1143,8 @@ class Event(ModelSQL, ModelView):
             else:
                 vevent.recurrence_id.value = event.recurrence\
                         .replace(tzinfo=tzlocal).astimezone(tzevent)
+        elif hasattr(vevent, 'recurrence-id'):
+            del vevent.recurrence_id
         if event.status:
             if not hasattr(vevent, 'status'):
                 vevent.add('status')
@@ -1185,6 +1176,10 @@ class Event(ModelSQL, ModelView):
         elif hasattr(vevent, 'location'):
             del vevent.location
 
+        if not hasattr(vevent, 'transp'):
+            vevent.add('transp')
+        vevent.transp.value = event.transp.upper()
+
         if event.organizer:
             if not hasattr(vevent, 'organizer'):
                 vevent.add('organizer')
@@ -1353,7 +1348,7 @@ class Attendee(ModelSQL, ModelView):
     _name = 'calendar.attendee'
 
     email = fields.Char('Email', required=True, states={
-        'readonly': 'active_id > 0',
+        'readonly': Greater(Eval('active_id', 0), 0),
         })
     status = fields.Selection([
         ('', ''),
@@ -1613,16 +1608,24 @@ class EventAttendee(ModelSQL, ModelView):
 EventAttendee()
 
 
-class RDate(ModelSQL, ModelView):
-    'Recurrence Date'
+class Date(ModelSQL, ModelView):
+    'Calendar Date'
     _description = __doc__
-    _name = 'calendar.rdate'
+    _name = 'calendar.date'
     _rec_name = 'datetime'
 
     date = fields.Boolean('Is Date', help='Ignore time of field "Date", ' \
             'but handle as date only.')
     datetime = fields.DateTime('Date', required=True)
 
+    def init(self, cursor, module_name):
+        # Migration from 1.4: calendar.rdate renamed to calendar.date
+        old_table = 'calendar_rdate'
+        if TableHandler.table_exist(cursor, old_table):
+            TableHandler.table_rename(cursor, old_table, self._table)
+
+        return super(Date, self).init(cursor, module_name)
+
     def _date2update(self, cursor, user, date, context=None):
         res = {}
         res['date'] = date.date
@@ -1658,7 +1661,7 @@ class RDate(ModelSQL, ModelView):
 
         :param cursor: the database cursor
         :param user: the user id
-        :param date: a BrowseRecord of calendar.rdate or
+        :param date: a BrowseRecord of calendar.date or
             calendar.exdate
         :param context: the context
         :return: a datetime
@@ -1670,21 +1673,30 @@ class RDate(ModelSQL, ModelView):
             res = date.datetime.replace(tzinfo=tzlocal).astimezone(tzutc)
         return res
 
-RDate()
+Date()
 
 
 class EventRDate(ModelSQL, ModelView):
     'Recurrence Date'
     _description = __doc__
     _name = 'calendar.event.rdate'
-    _inherits = {'calendar.rdate': 'calendar_rdate'}
+    _inherits = {'calendar.date': 'calendar_date'}
     _rec_name = 'datetime'
 
-    calendar_rdate = fields.Many2One('calendar.rdate', 'Calendar RDate',
+    calendar_date = fields.Many2One('calendar.date', 'Calendar Date',
             required=True, ondelete='CASCADE', select=1)
     event = fields.Many2One('calendar.event', 'Event', ondelete='CASCADE',
             select=1, required=True)
 
+    def init(self, cursor, module_name):
+        # Migration from 1.4: calendar_rdate renamed to calendar_date
+        table = TableHandler(cursor, self, module_name)
+        old_column = 'calendar_rdate'
+        if table.column_exist(old_column):
+            table.column_rename(old_column, 'calendar_date')
+
+        return super(EventRDate, self).init(cursor, module_name)
+
     def create(self, cursor, user, values, context=None):
         event_obj = self.pool.get('calendar.event')
         if values.get('event'):
@@ -1709,11 +1721,11 @@ class EventRDate(ModelSQL, ModelView):
 
     def delete(self, cursor, user, ids, context=None):
         event_obj = self.pool.get('calendar.event')
-        rdate_obj = self.pool.get('calendar.rdate')
+        rdate_obj = self.pool.get('calendar.date')
         if isinstance(ids, (int, long)):
             ids = [ids]
         event_rdates = self.browse(cursor, user, ids, context=context)
-        rdate_ids = [a.calendar_rdate.id for a in event_rdates]
+        rdate_ids = [a.calendar_date.id for a in event_rdates]
         event_ids = [x.event.id for x in event_rdates]
         if event_ids:
             # Update write_date of event
@@ -1724,15 +1736,15 @@ class EventRDate(ModelSQL, ModelView):
         return res
 
     def _date2update(self, cursor, user, date, context=None):
-        date_obj = self.pool.get('calendar.rdate')
+        date_obj = self.pool.get('calendar.date')
         return date_obj._date2update(cursor, user, date, context=context)
 
     def date2values(self, cursor, user, date, context=None):
-        date_obj = self.pool.get('calendar.rdate')
+        date_obj = self.pool.get('calendar.date')
         return date_obj.date2values(cursor, user, date, context=context)
 
     def date2date(self, cursor, user, date, context=None):
-        date_obj = self.pool.get('calendar.rdate')
+        date_obj = self.pool.get('calendar.date')
         return date_obj.date2date(cursor, user, date, context=context)
 
 EventRDate()
@@ -1788,8 +1800,8 @@ class RRule(ModelSQL, ModelView):
     def __init__(self):
         super(RRule, self).__init__()
         self._sql_constraints += [
-            ('until_count',
-                'CHECK(until IS NULL OR count IS NULL)',
+            ('until_count_only_one',
+                'CHECK(until IS NULL OR count IS NULL OR count = 0)',
                 'Only one of "until" and "count" can be set!'),
         ]
         self._constraints += [
@@ -1815,6 +1827,12 @@ class RRule(ModelSQL, ModelView):
             'invalid_bysetpos': 'Invalid "By Position"',
         })
 
+    def init(self, cursor, module_name):
+        # Migrate from 1.4: unit_count replaced by until_count_only_one
+        table  = TableHandler(cursor, self, module_name)
+        table.drop_constraint('until_count')
+        return super(RRule, self).init(cursor, module_name)
+
     def check_bysecond(self, cursor, user, ids):
         for rule in self.browse(cursor, user, ids):
             if not rule.bysecond:
