@@ -36,6 +36,7 @@ class Collection(ModelSQL, ModelView):
             return calendar_obj.get_name(cursor, user, calendar, context=context)
         return False
 
+    @Cache('webdav_collection.event')
     def event(self, cursor, user, uri, calendar_id=False, context=None):
         '''
         Return the event id in the uri or False
@@ -65,8 +66,6 @@ class Collection(ModelSQL, ModelView):
                 return event_ids[0]
         return False
 
-    event = Cache('webdav_collection.event')(event)
-
     def _caldav_filter_domain_calendar(self, cursor, user, filter, context=None):
         '''
         Return a domain for caldav filter on calendar
@@ -212,6 +211,21 @@ class Collection(ModelSQL, ModelView):
         return super(Collection, self).get_resourcetype(cursor, user, uri,
                 context=context, cache=cache)
 
+    def get_displayname(self, cursor, user, uri, context=None, cache=None):
+        calendar_obj = self.pool.get('calendar.calendar')
+        if uri in ('Calendars', 'Calendars/'):
+            return 'Calendars'
+        calendar_id = self.calendar(cursor, user, uri, context=context)
+        if calendar_id:
+            if not (uri[10:].split('/', 1) + [None])[1]:
+                return calendar_obj.browse(cursor, user, calendar_id,
+                        context=context).rec_name
+            return uri.split('/')[-1]
+        elif self.calendar(cursor, user, uri, ics=True, context=context):
+            return uri.split('/')[-1]
+        return super(Collection, self).get_displayname(cursor, user, uri,
+                context=context, cache=cache)
+
     def get_contenttype(self, cursor, user, uri, context=None, cache=None):
         if self.event(cursor, user, uri, context=context) \
                 or self.calendar(cursor, user, uri, ics=True, context=context):
@@ -604,4 +618,38 @@ class Collection(ModelSQL, ModelView):
         return super(Collection, self).exists(cursor, user, uri, context=context,
                 cache=cache)
 
+    def current_user_privilege_set(self, cursor, user, uri, context=None,
+            cache=None):
+        '''
+        Return the privileges of the current user for uri
+        Privileges ares: create, read, write, delete
+
+        :param cursor: the database cursor
+        :param user: the user id
+        :param uri: the uri
+        :param context: the context
+        :param cache: the cache
+        :return: a list of privileges
+        '''
+        calendar_obj = self.pool.get('calendar.calendar')
+
+        if uri in ('Calendars', 'Calendars/'):
+            return ['create', 'read', 'write', 'delete']
+        if uri and uri.startswith('Calendars/'):
+            calendar_id = self.calendar(cursor, user, uri, context=context)
+            if calendar_id:
+                calendar = calendar_obj.browse(cursor, user, calendar_id,
+                        context=context)
+                if user == calendar.owner.id:
+                    return ['create', 'read', 'write', 'delete']
+                res = []
+                if user in (x.id for x in calendar.read_users):
+                    res.append('read')
+                if user in (x.id for x in calendar.write_users):
+                    res.extend(['read', 'write', 'delete'])
+                return res
+            return []
+        return super(Collection, self).current_user_privilege_set(cursor, user,
+                uri, context=context, cache=cache)
+
 Collection()
